/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2007 University of Washington
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "ns3/debug.h"
#include "fw-ascii-trace.h"
#include "ns3/trace-root.h"
#include "ns3/trace-context.h"
#include "ns3/simulator.h"
#include "ns3/nstime.h"
#include "ns3/node-list.h"
#include "ns3/packet.h"
#include "ns3/queue.h"
#include "ns3/internet-node.h"
#include "ns3/ipv4.h"

NS_DEBUG_COMPONENT_DEFINE ("AsciiTraceFramework");

using namespace ns3;

AsciiTraceFramework::AsciiTraceFramework ()
: m_os ()
{
  NS_DEBUG("AsciiTraceFramework::AsciiTraceFramework ()");
}

AsciiTraceFramework::~AsciiTraceFramework ()
{
  NS_DEBUG("AsciiTraceFramework::~AsciiTraceFramework ()");
  m_os.close ();
}

  void
AsciiTraceFramework::Open (std::string filename)
{
  NS_DEBUG("AsciiTraceFramework::Open (" << filename << ")");

  m_os.open (filename.c_str ());

  if (m_os.fail())
    {
      NS_FATAL_ERROR("Can't open file " << filename);
    }
}

  void
AsciiTraceFramework::TraceQueues (void)
{
  NS_DEBUG("AsciiTraceFramework::TraceQueues ()");

  TraceRoot::Connect ("/nodes/*/ipv4/interfaces/*/netdevice/queue/*",
                      MakeCallback (&AsciiTraceFramework::LogQueue, this));
}

  void 
AsciiTraceFramework::LogQueue (TraceContext const &context, Packet const &p)
{
  NS_DEBUG("AsciiTraceFramework::LogQueue ()");

  if (!m_os.is_open()) return;

  enum Queue::TraceType type;
  context.Get (type);
  switch (type) 
    {
    case Queue::ENQUEUE:
      m_os << "+ ";
      break;
    case Queue::DEQUEUE:
      m_os << "- ";
      break;
    case Queue::DROP:
      m_os << "d ";
      break;
    }
  m_os << Simulator::Now ().GetSeconds () << " ";
  NodeList::NodeIndex nodeIndex;
  context.Get (nodeIndex);
  m_os << "node=" << NodeList::GetNode (nodeIndex)->GetId () << " ";
  Ipv4::InterfaceIndex interfaceIndex;
  context.Get (interfaceIndex);
  m_os << "interface=" << interfaceIndex << " ";
  m_os << "pkt-uid=" << p.GetUid () << " ";
  m_os << std::endl;
}

  void
AsciiTraceFramework::TraceNetDeviceRx (void)
{
  NS_DEBUG("AsciiTraceFramework::TraceNetDeviceRx ()");

  TraceRoot::Connect ("/nodes/*/ipv4/interfaces/*/netdevice/rx",
                      MakeCallback (&AsciiTraceFramework::LogNetDeviceRx, this));
}

  void 
AsciiTraceFramework::LogNetDeviceRx (TraceContext const &context, Packet &p)
{
  NS_DEBUG("AsciiTraceFramework::LogNetDeviceRx ()");

  if (!m_os.is_open()) return;

  m_os << "r " << Simulator::Now ().GetSeconds () << " ";
  NodeList::NodeIndex nodeIndex;
  context.Get (nodeIndex);
  m_os << "node=" << NodeList::GetNode (nodeIndex)->GetId () << " ";
  Ipv4::InterfaceIndex interfaceIndex;
  context.Get (interfaceIndex);
  m_os << "interface=" << interfaceIndex << " ";
  m_os << "pkt-uid=" << p.GetUid () << " ";
  m_os << std::endl;  
}
