/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2007 University of Washington
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Craig Dowell <craigdo@ee.washington.edu>
 */

#ifndef FW_BASE_H
#define FW_BASE_H

#include "ns3/debug.h"
#include "ns3/simulator.h"
#include "ns3/object-container.h"

namespace ns3 {

/**
 * \brief Framework Base Class
 *
 * This class represents the base class for an Ns-3 simulation framework.
 * For each generic usage scenario, this class is specialized to provide
 * an implementation of a generic application in that problem space.  A
 * user will further specialize the generic application (cf fw-simple) 
 * to construct his or her simulation application.
 *
 * The purpose of this particular class is to provide the functionality 
 * and plumbing that will need to exist for ALL generic simulations.
 */

class SimulationFrameworkBase {
public:
  SimulationFrameworkBase ();
  virtual ~SimulationFrameworkBase ();

  virtual void Run (void);

protected:
//
// These are the generic simulation methods.  We expect all simulations
// to go through the same sequence of events.
//
  virtual void InitStaticEnv (void);
  virtual void InitDynamicEnv (void);
  virtual void InitTopology (void);

  virtual void PreSimulation (void);
  virtual void Simulate (void);
  virtual void PostSimulation (void);

  virtual void CleanupTopology (void);
  virtual void CleanupDynamicEnv (void);
  virtual void CleanupStaticEnv (void);

  virtual void CreateIPAddress (void);
  virtual void CreateMACAddress (void);

  ObjectContainer m_objectContainer;

private:
  void InitBase (void);
  void CleanupBase (void);
};

} // namespace ns3

#endif /* FW_BASE_H */
